/* #region: Fonts */

/* noto-sans-display-500 - latin */
@font-face {
  font-family: "Noto Sans Display";
  font-style: normal;
  font-weight: 500;
  src:
    local(""),
    url("/static/fonts/noto-sans-display-latin-500.woff2") format("woff2"); /* Chrome 26+, Opera 23+, Firefox 39+ */
}

/* noto-sans-display-500 - latin italic */
@font-face {
  font-family: "Noto Sans Display";
  font-style: italic;
  font-weight: 500;
  src:
    local(""),
    url("/static/fonts/noto-sans-display-latin-500italic.woff2") format("woff2"); /* Chrome 26+, Opera 23+, Firefox 39+ */
}

/* fira-code-600 - latin */
@font-face {
  font-family: "Fira Code";
  font-style: normal;
  font-weight: 500;
  src:
    local(""),
    url("/static/fonts/fira-code-v12-latin-500.woff2") format("woff2"); /* Chrome 26+, Opera 23+, Firefox 39+ */
}

@font-face {
  font-family: "Fira Code";
  font-style: normal;
  font-weight: 600;
  src:
    local(""),
    url("/static/fonts/fira-code-v12-latin-600.woff2") format("woff2"); /* Chrome 26+, Opera 23+, Firefox 39+ */
}

/* fira-code-700 - latin */
@font-face {
  font-family: "Fira Code";
  font-style: normal;
  font-weight: 700;
  src:
    local(""),
    url("/static/fonts/fira-code-v12-latin-700.woff2") format("woff2"); /* Chrome 26+, Opera 23+, Firefox 39+ */
}

/* #endregion */

/* #region: Media Queries */

@media screen and (width <= 430px) {
  body {
    /* biome-ignore lint/complexity/noImportantStyles: TODO */
    margin-left: 10px !important;
    /* biome-ignore lint/complexity/noImportantStyles: TODO */
    margin-right: 10px !important;
  }

  .podcast-player {
    /* biome-ignore lint/complexity/noImportantStyles: TODO */
    margin-left: 10px !important;
    /* biome-ignore lint/complexity/noImportantStyles: TODO */
    margin-right: 10px !important;
  }
}

/* #endregion */

/* #region: General */

body {
  font-family:
    "Noto Sans Display", -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", sans-serif,
    "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol";
  font-size: 14px;
  font-weight: 500;
  background-color: #1a1a1a;
  color: #c8c8c8;
  line-height: 1.5;
  margin-top: 20px;
  margin-left: 23px;
  margin-right: 23px;
}

.contained-content {
  max-width: 700px;
}

h1,
h2 {
  font-family: "Fira Code", Consolas, "Lucida Console", monospace;
  line-height: 1.2;
  color: #f0f0f0;
}

h1 {
  font-size: 40px;
  font-weight: 700;
  margin: 8px 0 4px;
}

h2 {
  font-size: 24px;
  font-weight: 600;
  margin: 20px 0 2px;
}

h3,
h4 {
  font-family:
    "Noto Sans Display", -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", sans-serif,
    "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol";
  color: #c8c8c8;
  font-weight: 500;
  margin-top: 4px;
  margin-bottom: 6px;
  font-size: 18px;
}

h4 {
  font-style: italic;
}

p {
  margin-top: 6px;
  margin-bottom: 10px;
}

ol {
  margin: 0;
}

a:link {
  color: #008080;
}

a:visited {
  color: #885560;
}

::selection {
  color: #e6e6e6;
  background: #008080;
}

.active {
  color: #fff;
  display: inline;
}

header {
  font-size: smaller;
  margin: 10px 0 20px;
}

footer {
  font-size: smaller;
  margin: 20px 0 16px;
}

.last-generated {
  display: none;
}

.breadcrumb {
  margin: 0;
}

/* #endregion */

/* #region: File List, breadcrumbs */

.file-list {
  margin-top: 10px;
}

.file-list,
.breadcrumb {
  border-radius: 4px;
  display: list-item;
  list-style-type: none;
  padding: 8px;
  background-color: #101010;
  font-family: "Fira Code", Consolas, "Lucida Console", monospace;
  font-weight: 500;
  width: auto;
  white-space: nowrap; /* Prevent line breaks */
  overflow-x: auto; /* Enable horizontal scrolling */
}

.file-list::-webkit-scrollbar {
  height: 8px; /* Customize scrollbar height */
}

.file-list::-webkit-scrollbar-thumb {
  background-color: #888; /* Color of the scrollbar */
  border-radius: 4px; /* Rounded edges */
}

.file-list::-webkit-scrollbar-thumb:hover {
  background-color: #555; /* Darker color on hover */
}

/* #endregion */

/* #region: Podcast Episode List */

#podcast-episode-list li {
  /* background: #c8c8c8; */

  /* color: #c8c8c8; */
  border-bottom: 1px solid #ababab;
  cursor: pointer;
  white-space: nowrap;
  overflow: hidden;
}

#podcast-episode-list li:hover {
  background: #008080;
}

/*
#podcast-episode-list li:nth-child(odd) {
  background: #bbbbbb;
} */

#podcast-episode-list a:link {
  color: #c8c8c8;
  margin-top: 10px;
  font-style: unset;
  text-decoration: unset;
}

#podcast-episode-list a:visited {
  color: #c8c8c8;
  font-style: unset;
  text-decoration: unset;
}

/* #endregion */

/* #region: Button, selectors */
.button,
button,
input[type="submit"],
input[type="reset"],
input[type="button"] {
  display: inline-block;
  height: 30px;
  padding: 0 15px;
  color: #bbb;
  text-align: center;
  font-size: 11px;
  font-weight: 600;
  line-height: 30px;
  letter-spacing: 0.1rem;
  text-transform: uppercase;
  text-decoration: none;
  white-space: nowrap;
  background-color: #004040;
  border-radius: 4px;
  border: 1px solid #bbb;
  cursor: pointer;
  box-sizing: border-box;
  min-width: 100px;
}

.button:hover,
button:hover,
input[type="submit"]:hover,
input[type="reset"]:hover,
input[type="button"]:hover,
.button:focus,
button:focus,
input[type="submit"]:focus,
input[type="reset"]:focus,
input[type="button"]:focus {
  color: #333;
  border-color: #888;
  background-color: #008080;
  outline: 0;
}

/* Forms */
input[type="email"],
input[type="number"],
input[type="search"],
input[type="text"],
input[type="tel"],
input[type="url"],
input[type="password"],
textarea,
select {
  font-family: "Noto Sans Display", -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", sans-serif;
  background-color: #464646;
  border: 1px solid #d1d1d1;
  color: #bbb;
  border-radius: 4px;
  box-shadow: none;
  box-sizing: border-box;
  width: 250px;
  margin-top: 3px;
  margin-bottom: 3px;
}

input[type="email"],
input[type="number"],
input[type="search"],
input[type="text"],
input[type="tel"],
input[type="url"],
input[type="password"],
textarea {
  appearance: none; /* Remove default styling */
  height: 24px;
  padding: 6px 10px; /* The 6px vertically centers text on FF, ignored by Webkit */
}

select {
  height: 26px;
  padding: 2px 10px; /* The 6px vertically centers text on FF, ignored by Webkit */
}

textarea {
  min-height: 10px;
  padding-top: 1px;
  padding-bottom: 1px;
}

input[type="email"]:focus,
input[type="number"]:focus,
input[type="search"]:focus,
input[type="text"]:focus,
input[type="tel"]:focus,
input[type="url"]:focus,
input[type="password"]:focus,
textarea:focus,
select:focus {
  border: 1px solid #008080;
  outline: 0;
}

label,
legend {
  display: block;
  margin-bottom: 0.5rem;
  font-weight: 600;
}

fieldset {
  padding: 0;
  border-width: 0;
}

input[type="checkbox"],
input[type="radio"] {
  display: inline;
}

label > .label-body {
  display: inline-block;
  margin-left: 0.5rem;
  font-weight: normal;
}

.custom-select {
  position: relative;
  width: 200px;
}

.custom-select select {
  appearance: none;
  width: 100%;
  padding-left: 5px;
  border: 1px solid #bbb;
  border-radius: 5px;
  background-color: #333;
  font-size: 12px;
  cursor: pointer;
}

/* Dropdown arrow */
.custom-select::after {
  content: "▼"; /* Unicode down arrow */
  position: absolute;
  right: 10px;
  top: 50%;
  transform: translateY(-50%);
  pointer-events: none;
  font-size: 14px;
  color: #bbb;
}

/* Hover and focus styles */
.custom-select select:hover,
.custom-select select:focus {
  border-color: #008080;
}

/* #endregion */

/* #region: Table */

table {
  margin-top: 6px;
  margin-bottom: 2px;
  border: 1px solid #ababab;
  border-collapse: collapse;
}

tr {
  border: 2px solid #ababab;
  border-bottom: 2px solid #ababab;
}

th,
td {
  padding: 2px 10px;
  text-align: left;
  width: fit-content;
}

thead th {
  background-color: #008080;
  color: #fff;
  text-align: center;
}

/* #endregion */

/* #region: Health Table */

.health-table table {
  width: 100%;
  font-size: 11px;
  margin-top: 2px;
  margin-bottom: 2px;
  border: 1px solid #ababab;
  border-collapse: collapse;
}

.health-table tr {
  border: 1px solid #ababab;
  border-bottom: 1px solid #ababab;
}

.health-table th,
.health-table td {
  padding: 2px 4px;
  text-align: left;
}

.health-table td:first-child {
  width: 1px;
}

/* #endregion */

/* #region: Podcast Player */

.podcast-player-offset,
.podcast-player-footer {
  height: 105px;
}

.podcast-player-footer {
  position: fixed;
  left: 0;
  bottom: 0;
  width: 100%;
  background-color: #1a1a1a;
}

.podcast-player {
  margin-left: 23px;
  margin-right: 23px;
  max-width: 700px;
}

#podcast-audio-player {
  width: 100%;
}

#podcast-player-cover-container {
  display: inline-block;
  max-height: 80px;
  max-width: 80px;
  height: 100%;
  width: 100%;
}

#podcast-player-cover {
  margin-left: 5px;
  display: block;
  height: 100%;
  width: 100%;
}

#episode-name-and-player-container {
  display: inline-block;
  vertical-align: bottom;
  margin-left: 10px;
  width: calc(100% - 105px);
}

.podcast-player-text {
  margin-top: 0;
  margin-bottom: 1px;
  white-space: nowrap;
  overflow: hidden;
}

/* #endregion */

/* #region: Webkit Audio */

audio {
  height: 25px;
  margin-bottom: 3px;
  margin-top: 3px;
}

audio::-webkit-media-controls-enclosure,
audio::-webkit-media-controls-panel,
audio::-webkit-media-controls-play-button,
audio::-webkit-media-controls-volume-slider,
audio::-webkit-media-controls-mute-button,
audio::-webkit-media-controls-timeline,
audio::-webkit-media-controls-current-time-display,
audio::-webkit-media-controls-time-remaining-display,
audio::-webkit-time-display {
  border-radius: 4px;
  text-shadow: none;
}

audio::-webkit-media-controls-enclosure {
  height: 25px;
}

audio::-webkit-media-controls-panel,
video::-webkit-media-controls-panel {
  display: flex;
  flex-direction: row;
  align-items: center;

  /* We use flex-start here to ensure that the play button is visible even
   * if we are too small to show all controls.
   */
  justify-content: flex-start;
  user-select: none;
  position: relative;
  width: 100%;
  z-index: 0;
  overflow: hidden;
  text-align: right;
  bottom: auto;
  height: 25px;

  /* background-color: rgba(20, 20, 20, 0.8); */
  border-radius: 5px;

  /* The duration is also specified in MediaControlElements.cpp and LayoutTests/media/media-controls.js */
  transition: opacity 0.3s;
}

audio::-webkit-media-controls-panel {
  /* background-color: #101010; */

  /* padding: 0px; */
  border: none;
  margin: 0;
}

audio::-webkit-media-controls-play-button {
  background-color: #008080;
  color: #fff;
  height: 18px;
  width: 10px;
}

audio::-webkit-media-controls-current-time-display,
audio::-webkit-media-controls-time-remaining-display,
audio::-webkit-time-display {
  text-shadow: none;
}

/* #endregion */
