<!DOCTYPE html>
<html lang="en">

<head>
    <meta charset="utf-8">
    <title>
        {{ app_config['web_page']['title']|e }}
    </title>
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <meta http-equiv="X-Clacks-Overhead" content="GNU Terry Pratchett">
    <link rel="preload" href="/static/fonts/noto-sans-display-latin-500.woff2" as="font" type="font/woff2" crossorigin>
    <link rel="preload" href="/static/fonts/fira-code-v12-latin-500.woff2" as="font" type="font/woff2" crossorigin>
    <link rel="preload" href="/static/fonts/fira-code-v12-latin-600.woff2" as="font" type="font/woff2" crossorigin>
    <link rel="preload" href="/static/fonts/fira-code-v12-latin-700.woff2" as="font" type="font/woff2" crossorigin>
    <link rel="preload" href="/static/fonts/noto-sans-display-latin-500italic.woff2" as="font" type="font/woff2"
        crossorigin>
    <link rel="stylesheet" href="/static/main.css">
    <link rel="icon" type="image/x-icon" href="/favicon.ico">
</head>

<body>
    <main class="contained-content">
        <h1>{{ app_config['web_page']['title']|e }}</h1>
        {{ header | safe }}
        <p>Guides for some popular podcast applications.</p>
        <h2>Pocket Casts</h2>
        <h4>iOS, Android</h4>
        <ol>
            <li>Tap Discover</li>
            <li>Paste URL into the search</li>
            <li>Tap Search</li>
            <li>Tap the Plus button</li>
        </ol>

        <h2>Apple Podcasts</h2>
        <h4>iOS</h4>
        <ol>
            <li>Tap Library</li>
            <li>Tap &quot;...&quot;</li>
            <li>Tap Add a Show by URL</li>
            <li>Paste URL</li>
            <li>Tap Follow</li>
        </ol>

        <h2>YouTube Music</h2>
        <h4>Android, iOS, Web</h4>
        <ol>
            <li>Tap Library</li>
            <li>Scroll the library categories for &quot;Podcasts&quot;</li>
            <li>Tap Add podcast</li>
            <li>Tap Add podcast by RSS feed</li>
            <li>Paste URL</li>
            <li>Tap Add</li>
        </ol>


        <h2>AntennaPod</h2>
        <h4>Android</h4>
        <ol>
            <li>Tap ☰</li>
            <li>Tap Add Podcast</li>
            <li>Tap Add Podcast by RSS address</li>
            <li>Tap Add</li>
        </ol>

        <h2>PodcastAddict</h2>
        <h4>Android</h4>
        <ol>
            <li>Tap +</li>
            <li>Tap the RSS button</li>
            <li>Paste URL</li>
            <li>Tap Add</li>
        </ol>

        <h2>Overcast</h2>
        <h4>iOS</h4>
        <ol>
            <li>Tap the Search button</li>
            <li>Tap Add URL</li>
            <li>Paste URL</li>
            <li>Tap Done</li>
        </ol>

        <h2>Podbean</h2>
        <h4>Android, iOS, God I hate this app</h4>
        <ol>
            <li>Tap Search</li>
            <li>Tap +Feed</li>
            <li>Paste URL</li>
            <li>Tap Done</li>
        </ol>

        <h2>gPodder</h2>
        <h4>Linux, MacOS, Windows</h4>
        <ol>
            <li>Subscriptions</li>
            <li>Add Podcast by URL</li>
            <li>Paste URL</li>
            <li>Tap Add</li>
        </ol>

        <h2>VLC</h2>
        <h4>Linux, MacOS, Windows</h4>
        <ol>
            <li>View</li>
            <li>Playlist</li>
            <li>Podcasts</li>
            <li>Click the Plus next to Podcasts</li>
            <li>Paste URL</li>
            <li>Click OK</li>
        </ol>
    </main>
</body>

</html>
